@@ -1,31 +1,34 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
-** This file is part of the QtQuick.Dialogs module of the Qt Toolkit.
+** This file is part of the Qt Quick Dialogs module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL3$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or later as published by the Free
+** Software Foundation and appearing in the file LICENSE.GPL included in
+** the packaging of this file. Please review the following information to
+** ensure the GNU General Public License version 2.0 requirements will be
+** met: http://www.gnu.org/licenses/gpl-2.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -62,6 +65,7 @@ QQuickAbstractDialog::QQuickAbstractDialog(QObject *parent)
         hasCapability(QPlatformIntegration::WindowManagement))
     , m_hasAspiredPosition(false)
     , m_visibleChangedConnected(false)
+    , m_dialogHelperInUse(false)
 {
 }
 
@@ -73,110 +77,129 @@ void QQuickAbstractDialog::setVisible(bool v)
 {
     if (m_visible == v) return;
     m_visible = v;
-    if (helper()) {
-        qCDebug(lcWindow) << "via helper" << helper();
-        if (v) {
-            Qt::WindowFlags flags = Qt::Dialog;
-            if (!title().isEmpty())
-                flags |= Qt::WindowTitleHint;
-            m_visible = helper()->show(flags, m_modality, parentWindow());
-        } else {
-            helper()->hide();
-        }
-    } else {
-        // Pure QML implementation: wrap the contentItem in a window, or fake it
-        if (!m_dialogWindow && m_contentItem) {
-            if (m_hasNativeWindows)
-                m_dialogWindow = m_contentItem->window();
-            // An Item-based dialog implementation doesn't come with a window, so
-            // we have to instantiate one iff the platform allows it.
-            if (!m_dialogWindow && m_hasNativeWindows) {
-                QQuickWindow *win = new QQuickWindow;
-                ((QObject *)win)->setParent(this); // memory management only
-                win->setFlags(Qt::Dialog);
-                m_dialogWindow = win;
-                m_contentItem->setParentItem(win->contentItem());
-                QSize minSize = QSize(m_contentItem->implicitWidth(), m_contentItem->implicitHeight());
-                QVariant minHeight = m_contentItem->property("minimumHeight");
-                if (minHeight.isValid()) {
-                    if (minHeight.toInt() > minSize.height())
-                        minSize.setHeight(minHeight.toDouble());
-                    connect(m_contentItem, SIGNAL(minimumHeightChanged()), this, SLOT(minimumHeightChanged()));
-                }
-                QVariant minWidth = m_contentItem->property("minimumWidth");
-                if (minWidth.isValid()) {
-                    if (minWidth.toInt() > minSize.width())
-                        minSize.setWidth(minWidth.toInt());
-                    connect(m_contentItem, SIGNAL(minimumWidthChanged()), this, SLOT(minimumWidthChanged()));
-                }
-                m_dialogWindow->setMinimumSize(minSize);
-                connect(win, SIGNAL(widthChanged(int)), this, SLOT(windowGeometryChanged()));
-                connect(win, SIGNAL(heightChanged(int)), this, SLOT(windowGeometryChanged()));
-                qCDebug(lcWindow) << "created window" << win;
-            }
 
-            if (!m_dialogWindow) {
-                if (Q_UNLIKELY(!parentWindow())) {
-                    qWarning("cannot set dialog visible: no window");
+    if (m_dialogHelperInUse || v) {
+        // To show the dialog, we first check if there is a dialog helper that can be used
+        // and that show succeeds given the current configuration. Otherwise we fall back
+        // to use the pure QML version.
+        if (QPlatformDialogHelper *dialogHelper = helper()) {
+            if (v) {
+                Qt::WindowFlags flags = Qt::Dialog;
+                if (!title().isEmpty())
+                    flags |= Qt::WindowTitleHint;
+                if (dialogHelper->show(flags, m_modality, parentWindow())) {
+                    qCDebug(lcWindow) << "Show dialog using helper:" << dialogHelper;
+                    m_dialogHelperInUse = true;
+                    emit visibilityChanged();
                     return;
                 }
-                m_dialogWindow = parentWindow();
-
-                // If the platform does not support multiple windows, but the dialog is
-                // implemented as an Item, then try to decorate it as a fake window and make it visible.
-                if (!m_windowDecoration) {
-                    if (m_decorationComponent) {
-                        if (m_decorationComponent->isLoading())
-                            connect(m_decorationComponent, SIGNAL(statusChanged(QQmlComponent::Status)),
-                                    this, SLOT(decorationLoaded()));
-                        else
-                            decorationLoaded(); // do the reparenting of contentItem on top of it
-                    }
-                    // Window decoration wasn't possible, so just reparent it into the scene
-                    else {
-                        qCDebug(lcWindow) << "no window and no decoration";
-                        m_contentItem->setParentItem(parentWindow()->contentItem());
-                        m_contentItem->setZ(10000);
-                    }
-                }
+            } else {
+                qCDebug(lcWindow) << "Hide dialog using helper:" << dialogHelper;
+                dialogHelper->hide();
+                emit visibilityChanged();
+                return;
             }
         }
-        if (m_dialogWindow) {
-            // "grow up" to the size and position expected to achieve
-            if (!m_sizeAspiration.isNull()) {
-                if (m_hasAspiredPosition) {
-                    qCDebug(lcWindow) << "geometry aspiration" << m_sizeAspiration;
-                    m_dialogWindow->setGeometry(m_sizeAspiration);
-                } else {
-                    qCDebug(lcWindow) << "size aspiration" << m_sizeAspiration.size();
-                    if (m_sizeAspiration.width() > 0)
-                        m_dialogWindow->setWidth(m_sizeAspiration.width());
-                    if (m_sizeAspiration.height() > 0)
-                        m_dialogWindow->setHeight(m_sizeAspiration.height());
-                }
-                connect(m_dialogWindow, SIGNAL(xChanged(int)), this, SLOT(setX(int)));
-                connect(m_dialogWindow, SIGNAL(yChanged(int)), this, SLOT(setY(int)));
-                connect(m_dialogWindow, SIGNAL(widthChanged(int)), this, SLOT(setWidth(int)));
-                connect(m_dialogWindow, SIGNAL(heightChanged(int)), this, SLOT(setHeight(int)));
-                connect(m_contentItem, SIGNAL(implicitHeightChanged()), this, SLOT(implicitHeightChanged()));
-            }
-            if (!m_visibleChangedConnected) {
-                connect(m_dialogWindow, SIGNAL(visibleChanged(bool)), this, SLOT(visibleChanged(bool)));
-                m_visibleChangedConnected = true;
-            }
-        }
-        if (m_windowDecoration) {
-            m_windowDecoration->setProperty("dismissOnOuterClick", (m_modality == Qt::NonModal));
-            m_windowDecoration->setVisible(v);
-        } else if (m_dialogWindow) {
-            if (v) {
-                m_dialogWindow->setTransientParent(parentWindow());
-                m_dialogWindow->setTitle(title());
-                m_dialogWindow->setModality(m_modality);
-            }
-            m_dialogWindow->setVisible(v);
-        }
     }
+
+    qCDebug(lcWindow) << "Show/hide dialog using pure QML";
+    m_dialogHelperInUse = false;
+
+    // Pure QML implementation: wrap the contentItem in a window, or fake it
+    if (!m_dialogWindow && m_contentItem) {
+        if (v)
+            emit __maximumDimensionChanged();
+        if (m_hasNativeWindows)
+            m_dialogWindow = m_contentItem->window();
+        // An Item-based dialog implementation doesn't come with a window, so
+        // we have to instantiate one iff the platform allows it.
+        if (!m_dialogWindow && m_hasNativeWindows) {
+            QQuickWindow *win = new QQuickWindow;
+            ((QObject *)win)->setParent(this); // memory management only
+            win->setFlags(Qt::Dialog);
+            m_dialogWindow = win;
+            m_contentItem->setParentItem(win->contentItem());
+            QSize minSize = QSize(m_contentItem->implicitWidth(), m_contentItem->implicitHeight());
+            QVariant minHeight = m_contentItem->property("minimumHeight");
+            if (minHeight.isValid()) {
+                if (minHeight.toInt() > minSize.height())
+                    minSize.setHeight(minHeight.toDouble());
+                connect(m_contentItem, SIGNAL(minimumHeightChanged()), this, SLOT(minimumHeightChanged()));
+            }
+            QVariant minWidth = m_contentItem->property("minimumWidth");
+            if (minWidth.isValid()) {
+                if (minWidth.toInt() > minSize.width())
+                    minSize.setWidth(minWidth.toInt());
+                connect(m_contentItem, SIGNAL(minimumWidthChanged()), this, SLOT(minimumWidthChanged()));
+            }
+            m_dialogWindow->setMinimumSize(minSize);
+            connect(win, SIGNAL(widthChanged(int)), this, SLOT(windowGeometryChanged()));
+            connect(win, SIGNAL(heightChanged(int)), this, SLOT(windowGeometryChanged()));
+            qCDebug(lcWindow) << "created window" << win;
+        }
+
+        if (!m_dialogWindow) {
+            if (Q_UNLIKELY(!parentWindow())) {
+                qWarning("cannot set dialog visible: no window");
+                return;
+            }
+            m_dialogWindow = parentWindow();
+
+            // If the platform does not support multiple windows, but the dialog is
+            // implemented as an Item, then try to decorate it as a fake window and make it visible.
+            if (!m_windowDecoration) {
+                if (m_decorationComponent) {
+                    if (m_decorationComponent->isLoading())
+                        connect(m_decorationComponent, SIGNAL(statusChanged(QQmlComponent::Status)),
+                                this, SLOT(decorationLoaded()));
+                    else
+                        decorationLoaded(); // do the reparenting of contentItem on top of it
+                }
+                // Window decoration wasn't possible, so just reparent it into the scene
+                else {
+                    qCDebug(lcWindow) << "no window and no decoration";
+                    m_contentItem->setParentItem(parentWindow()->contentItem());
+                    m_contentItem->setZ(10000);
+                }
+            }
+        }
+    }
+    if (m_dialogWindow) {
+        // "grow up" to the size and position expected to achieve
+        if (!m_sizeAspiration.isNull()) {
+            if (m_hasAspiredPosition) {
+                qCDebug(lcWindow) << "geometry aspiration" << m_sizeAspiration;
+                m_dialogWindow->setGeometry(m_sizeAspiration);
+            } else {
+                qCDebug(lcWindow) << "size aspiration" << m_sizeAspiration.size();
+                if (m_sizeAspiration.width() > 0)
+                    m_dialogWindow->setWidth(m_sizeAspiration.width());
+                if (m_sizeAspiration.height() > 0)
+                    m_dialogWindow->setHeight(m_sizeAspiration.height());
+            }
+            connect(m_dialogWindow, SIGNAL(xChanged(int)), this, SLOT(setX(int)));
+            connect(m_dialogWindow, SIGNAL(yChanged(int)), this, SLOT(setY(int)));
+            connect(m_dialogWindow, SIGNAL(widthChanged(int)), this, SLOT(setWidth(int)));
+            connect(m_dialogWindow, SIGNAL(heightChanged(int)), this, SLOT(setHeight(int)));
+            connect(m_contentItem, SIGNAL(implicitHeightChanged()), this, SLOT(implicitHeightChanged()));
+        }
+        if (!m_visibleChangedConnected) {
+            connect(m_dialogWindow, SIGNAL(visibleChanged(bool)), this, SLOT(visibleChanged(bool)));
+            m_visibleChangedConnected = true;
+        }
+    }
+    if (m_windowDecoration) {
+        m_windowDecoration->setProperty("dismissOnOuterClick", (m_modality == Qt::NonModal));
+        m_windowDecoration->setVisible(v);
+    } else if (m_dialogWindow) {
+        if (v) {
+            m_dialogWindow->setTransientParent(parentWindow());
+            m_dialogWindow->setTitle(title());
+            m_dialogWindow->setModality(m_modality);
+        }
+        m_dialogWindow->setVisible(v);
+    }
+
     emit visibilityChanged();
 }
 
@@ -199,7 +222,7 @@ void QQuickAbstractDialog::decorationLoaded()
             m_windowDecoration->setProperty("content", contentVariant);
             connect(m_windowDecoration, SIGNAL(dismissed()), this, SLOT(reject()));
             ok = true;
-            qCDebug(lcWindow) << "using synthetic window decoration" << m_windowDecoration;
+            qCDebug(lcWindow) << "using synthetic window decoration" << m_windowDecoration << "from" << m_decorationComponent->url();
         } else {
             qWarning() << m_decorationComponent->url() <<
                 "cannot be used as a window decoration because it's not an Item";
@@ -339,6 +362,7 @@ int QQuickAbstractDialog::height() const
 int QQuickAbstractDialog::__maximumDimension() const
 {
     QScreen *screen = QGuiApplication::primaryScreen();
+    qCDebug(lcWindow) << "__maximumDimension checking screen" << screen << "geometry" << screen->availableVirtualGeometry();
     return (screen ?
                 qMin(screen->availableVirtualGeometry().width(), screen->availableVirtualGeometry().height()) :
                 480) * 9 / 10;
